class ItemNotFoundException(Exception):
    '''Raised when an item was not found'''
    pass

class InvalidParametersException(Exception):
    '''Raised when parameters where invalid'''
    pass

class DatabaseSaveFailedException(Exception):
    '''Raised when saving to database fails'''
    pass

class RoomHasDataException(Exception):
    '''Raised when a room that should be deleted has data left'''
    pass

class AbstractFunctionCallException(Exception):
    '''Raised when a abstract method is called'''
    pass

class NoSuchTypeException(Exception):
    '''Raised when a devicetype is used that does not exist'''
    pass