#!/usr/bin/python
# -*- coding: utf-8 -*-

WHITE = "000000"

RED = "FF0000"
GREEN = "00FF00"
BLUE = "0000FF"

LIGHT_RED = "FF7F7F"
LIGHT_GREEN = "9BFF70"
LIGHT_BLUE = "00FFFF"

COLOR_NAMES = {
    'grün': GREEN,
    'rot': RED,
    'blau': BLUE,
    'hellgrün': LIGHT_GREEN,
    'hellblau': LIGHT_BLUE,
    'hellrot': LIGHT_RED,
    'weiß': WHITE
}

COLORS = list(COLOR_NAMES.keys())